
# q-expansion of newform 550.6.b.b, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 550
weight = 6
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[177, [-1, 0]], [101, [1, 0]]]
aps_data = [[0, 2], [0, 6], [0, 0], [0, -27], [-121, 0], [0, -270], [0, -170], [952, 0], [0, 546], [62, 0], [-7560, 0], [0, 4593], [-6818, 0], [0, -6655], [0, 11210], [0, 9827], [-48292, 0], [17530, 0], [0, 17672], [-22912, 0], [0, 23926], [-52396, 0], [0, 3945], [-41958, 0], [0, 18801], [57406, 0], [0, -25264], [0, 39175], [-59990, 0], [0, -30097], [0, -57733], [302896, 0], [0, 65829], [-129052, 0], [-494814, 0], [27044, 0], [0, 126207], [0, -162862], [0, 67107], [0, -179968], [-413652, 0], [-112030, 0], [-317280, 0], [0, -247498], [0, -101016], [-471896, 0], [171136, 0], [0, 416286], [0, -603499], [1025018, 0], [0, 361404], [51556, 0], [-955890, 0], [-1809476, 0], [0, 488903], [0, 697275], [-1189310, 0], [510412, 0], [0, 999138], [-2614050, 0], [0, -900183], [0, 433614], [0, 367867], [1554048, 0], [0, 1023717], [0, 555331], [-1297348, 0], [0, 73342], [0, 1591179], [2146134, 0], [0, -1828043], [-751316, 0], [0, 1153546], [0, 2477964], [-3645972, 0], [0, 1911004], [2368074, 0], [0, -445313], [3582766, 0], [4077494, 0], [20820, 0], [4935146, 0], [-1278808, 0], [0, 1523647], [-3553040, 0], [0, 1938442], [2294202, 0], [0, -195684], [869814, 0], [0, -2875426], [0, 2282224], [-3156036, 0], [0, 1570602], [-7233876, 0], [7844020, 0], [0, -2973327], [6209714, 0], [-8890858, 0], [0, 1317127], [505174, 0], [0, 1265717], [0, 3513696], [0, -1107501], [-3700770, 0], [1183188, 0], [0, -6651139], [0, 919620], [0, -6761242], [169296, 0], [-5300706, 0], [0, 13267], [0, -4274548], [0, -900491], [-7279820, 0], [-5312976, 0], [-1037442, 0], [0, -7425718], [0, 5047680], [0, 5077235], [-18179388, 0], [11311454, 0], [0, -3781418], [0, -3923638], [0, -6925600], [-5211788, 0], [14547890, 0], [-25102634, 0], [16576936, 0], [0, -8105140], [0, -12435120], [-26791804, 0], [0, -2764317], [20122160, 0], [0, -10289161], [-9531198, 0], [-21374318, 0], [0, 6374023], [0, 15719551], [0, 8899755], [1995498, 0], [9092956, 0], [-24567894, 0], [0, -3101540], [0, -14418701], [-20705794, 0], [16802232, 0], [0, 10310120], [0, 11649286], [-26480796, 0], [0, 3904392], [0, -11548456], [24852058, 0], [0, 16965378], [0, 19993963], [0, 2261148], [29095152, 0], [25131488, 0], [19330238, 0], [0, -19796572], [44973406, 0], [0, -17884740], [0, -19155348], [0, 5974383], [27443364, 0], [0, 13713429], [0, 6609588], [32018032, 0], [0, -23911148]]
